__title__ = 'Restcord'
__author__ = 'JustMaffie'
__license__ = 'MIT'
__copyright__ = 'Copyright 2015-2017 JustMaffie'
__github__ = "https://github.com/JustMaffie/Restcord"
__major__ = "1"
__minor__ = "0"
__patch__ = "4"
__version__ = "{}.{}.{}".format(__major__, __minor__, __patch__)


# -*- coding: utf-8 -*-

"""
Discord API Wrapper
~~~~~~~~~~~~~~~~~~~

A basic wrapper for the Discord API.

:copyright: (c) 2015-2017 JustMaffie
:license: MIT, see LICENSE for more details.

"""
from .core import *
from .models import *
from .http import *
from .vars import *


import aiohttp
from .vars import __version__ as version
from .vars import __github__ as github

BASE_URL = "https://discordapp.com/api/v6"

class HTTPClient:
	def __init__(self, restcord):
		self.session = aiohttp.ClientSession()
		self.client = restcord

	def get_headers(self):
		headers = {}
		token = self.client.token
		if not self.client.selfbot:
			token = "Bot {}".format(token)
		headers['Authorization'] = token
		headers['User-Agent'] = "RestCord Python ({}, v{})".format(github, version)
		return headers

	async def get(self, endpoint):
		resp = await self.session.get(BASE_URL + endpoint, headers=self.get_headers())
		return await resp.json()

	async def post(self, endpoint, data=None):
		resp = await self.session.post(BASE_URL + endpoint, headers=self.get_headers(), json=data)
		return await resp.json()

	async def delete(self, endpoint):
		resp = await self.session.delete(BASE_URL + endpoint, headers=self.get_headers())
		return await resp.json()

	async def head(self, endpoint):
		resp = await self.session.head(BASE_URL + endpoint, headers=self.get_headers())
		return await resp.json()

	async def options(self, endpoint):
		resp = await self.session.options(BASE_URL + endpoint, headers=self.get_headers())
		return await resp.json()

	async def patch(self, endpoint, data=None):
		resp = await self.session.patch(BASE_URL + endpoint, headers=self.get_headers(), json=data)
		return await resp.json()


from .http import HTTPClient
from .models import *

def get_from_kwargs(kwargs, key):
	if not kwargs:
		return None
	if key not in kwargs:
		return None
	return kwargs[key]

class Restcord:

	def __init__(self, **kwargs):
		token = get_from_kwargs(kwargs, "token")
		if not token:
			raise RuntimeError("You must provide a token")
		selfbot = get_from_kwargs(kwargs, "selfbot")
		if not selfbot:
			selfbot = False
		self.token = token
		self.selfbot = selfbot
		self.http = HTTPClient(self)

	async def get_user(self, id):
		resp = await self.http.get("/users/{}".format(id))
		return User(**resp)

	async def get_user_me(self):
		resp = await self.http.get("/users/@me")
		return User(**resp)

	async def modify_user_me(self, data):
		resp = await self.http.patch("/users/@me", data)
		return User(**resp)

	async def get_guilds_me(self):
		resp = await self.http.get("/users/@me/guilds")
		guilds = []
		for guild in resp:
			guilds.append(Guild(**guild))
		return guilds

	async def leave_guild(self, id):
		await self.http.delete("/users/@me/guilds/{}".format(id))

	async def get_user_dms(self):
		resp = await self.http.get("/users/@me/channels")
		channels = []
		for chan in resp:
			channels.append(Channel(**chan))
		return channels

	async def create_dm(self, recipient_id):
		data = {recipient_id: recipient_id}
		resp = await self.http.post("/users/@me/channels")
		return Channel(**resp)

	async def get_channel(self, id):
		resp = await self.http.get("/channels/{}".format(id))
		return Channel(**resp)

	async def modify_channel(self, id, data):
		resp = await self.http.put("/channels/{}".format(id), data)
		return Channel(**resp)

	async def delete_channel(self, id):
		resp = await self.http.delete("/channels/{}".format(id))
		return Channel(**resp)

	async def get_messages(self, channel_id):
		resp = await self.http.get("/channels/{}/messages".format(channel_id))
		messages = []
		for msg in resp:
			messages.append(Message(**msg))
		return messages

	async def get_message(self, channel_id, message_id):
		resp = await self.http.get("/channels/{}/messages/{}".format(channel_id, message_id))
		return Message(**resp)

	async def send_message(self, channel_id, data):
		resp = await self.http.post("/channels/{}/messages".format(channel_id), data)
		return Message(**resp)

	async def add_reaction(self, channel_id, message_id, emoji):
		await self.http.put("/channels/{}/messages/{}/reactions/{}/@me".format(channel_id, message_id, emoji))

	async def remove_reaction(self, channel_id, message_id, emoji):
		await self.http.delete("/channels/{}/messages/{}/reactions/{}/@me".format(channel_id, message_id, emoji))

	async def remove_user_reaction(self, channel_id, message_id, emoji, user_id):
		await self.http.delete("/channels/{}/messages/{}/reactions/{}/{}".format(channel_id, message_id, emoji, user_id))

	async def get_reactions(self, channel_id, message_id, emoji):
		resp = await self.http.get("/channels/{}/messages/{}/reactions/{}".format(channel_id, message_id, emoji))
		users = []
		for user in resp:
			users.append(User(**user))
		return users

	async def delete_all_reactions(self, channel_id, message_id):
		await self.http.delete("/channels/{}/messages/{}/reactions".format(channel_id, message_id))

	async def edit_message(self, channel_id, message_id, data):
		resp = await self.http.patch("/channels/{}/messages/{}".format(channel_id, message_id), data)
		return Message(**resp)

	async def delete_message(self, channel_id, message_id):
		await self.http.delete("/channels/{}/messages/{}".format(channel_id, message_id))

	async def get_invites_channel(self, channel_id):
		resp = await self.http.get("/channels/{}/invites".format(channel_id))
		invites = []
		for invite in resp:
			invites.append(Invite(**invite))
		return invites

	async def create_invite(self, channel_id, data):
		resp = await self.http.post("/channels/{}/invites".format(channel_id), data)
		return Invite(**resp)

	async def send_typing(self, channel_id):
		await self.http.post("/channels/{}/typing".format(channel_id))

	async def get_pinned_messages(self, channel_id):
		resp = await self.http.get("/channels/{}/pins".format(channel_id))
		messages = []
		for msg in resp:
			messages.append(Message(**msg))
		return messages

	async def pin_message(self, channel_id, message_id):
		await self.http.put("/channels/{}/pins/{}".format(channel_id, message_id))

	async def unpin_message(self, channel_id, message_id):
		await self.http.delete("/channels/{}/pins/{}".format(channel_id, message_id))

	async def create_guild(self, data):
		resp = await self.http.post("/guilds", data)
		return Guild(**resp)

	async def get_guild(self, guild_id):
		resp = await self.http.get("/guilds/{}".format(guild_id))
		return Guild(**resp)

	async def modify_guild(self, guild_id, data=None):
		resp = await self.http.patch("/guilds/{}".format(guild_id), data)
		return Guild(**resp)

	async def delete_guild(self, guild_id):
		await self.http.delete("/guilds/{}".format(guild_id))

	async def get_guild_channels(self, guild_id):
		resp = await self.http.get("/guilds/{}/channels".format(guild_id))
		chans = []
		for channel in resp:
			chans.append(Channel(**channel))
		return chans

	async def create_guild_channel(self, guild_id, data):
		resp = await self.http.post("/guilds/{}/channels", data)
		return Channel(resp)

	async def get_member(self, guild_id, member_id):
		resp = await self.http.get("/guilds/{}/members/{}")
		return User(resp)

	async def get_members(self, guild_id):
		resp = await self.http.get("/guilds/{}/members")
		membs = []
		for m in resp:
			membs.append(Member(**m))
		return membs

	async def modify_guild_member(self, guild_id, member_id, data):
		await self.http.patch("/guilds/{}/members/{}".format(guild_id, member_id), data)

	async def set_nick_self(self, guild_id, nick):
		data = {nick: nick}
		await self.http.patch("/guilds/{}/members/@me/nick".format(guild_id), data)

	async def add_role(self, guild_id, member_id, role_id):
		await self.http.put("/guilds/{}/members/{}/roles/{}".format(guild_id, member_id, role_id))

	async def remove_role(self, guild_id, member_id, role_id):
		await self.http.delete("/guilds/{}/members/{}/roles/{}".format(guild_id, member_id, role_id))

	async def kick_member(self, guild_id, member_id):
		await self.http.delete("/guilds/{}/members/{}".format(guild_id, member_id))

	async def get_bans(self, guild_id):
		resp = await self.http.get("/guilds/{}/bans".format(guild_id))
		bans = []
		for ban in resp:
			bans.append(Ban(**ban))
		return bans

	async def ban_user(self, guild_id, member_id, delete_message_days):
		data = {}
		data['delete-message-days'] = delete_message_days
		await self.http.put("/guilds/{}/bans/{}".format(guild_id, member_id), data)

	async def unban(self, guild_id, member_id):
		await self.http.delete("/guilds/{}/bans/{}".format(guild_id, member_id))

	async def get_roles(self, guild_id):
		resp = await self.http.get("/guilds/{}/roles".format(guild_id))
		roles = []
		for role in resp:
			roles.append(Role(**role))
		return roles

	async def create_role(self, guild_id, data):
		resp = await self.http.post("/guilds/{}/roles".format(guild_id), data)
		return Role(**resp)

	async def modify_guild_roles(self, guild_id, data):
		resp = await self.http.patch("/guilds/{}/roles".format(guild_id), data)
		roles = []
		for role in resp:
			roles.append(Role(**role))
		return roles

	async def modify_guild_role(self, guild_id, role_id, data):
		resp = await self.http.patch("/guilds/{}/roles/{}".format(guild_id, role_id), data)
		return Role(**resp)

	async def remove_role(self, guild_id, role_id):
		await self.http.delete("/guilds/{}/roles/{}".format(guild_id, role_id))

	async def get_prune_count(self, guild_id, days):
		resp = await self.http.get("/guilds/{}/prune?days={}".format(guild_id, days))
		return resp['pruned']

	async def begin_prune(self, guild_id, days):
		data = {days:days}
		await self.http.post("/guilds/{}/prune".format(guild_id), data)

	async def get_invites_guild(self, guild_id):
		resp = await self.http.get("/guilds/{}/invites")
		invites = []
		for invite in resp:
			invites.append(Invite(**invite))
		return invites

	async def get_guild_integrations(self, guild_id):
		resp = await self.http.get("/guilds/{}/integrations")
		integrations = []
		for integration in resp:
			integrations.append(Integration(**integration))
		return integrations

	async def create_guild_integration(self, guild_id, data):
		await self.http.post("/guilds/{}/integrations".format(guild_id), data)

	async def modify_guild_integration(self, guild_id, integration_id, data):
		await self.http.patch("/guilds/{}/integrations/{}".format(guild_id, integration_id), data)

	async def delete_guild_integration(self, guild_id, integration_id):
		await self.http.delete("/guilds/{}/integrations/{}".format(guild_id, integration_id))

	async def sync_guild_integration(self, guild_id, integration_id):
		await self.http.post("/guilds/{}/integrations/{}/sync".format(guild_id, integration_id))

	async def get_guild_embed(self, guild_id):
		resp = await self.http.get("/guilds/{}/embed")
		return GuildEmbed(**resp)

	async def modify_guild_embed(self, guild_id, data):
		resp = await self.http.patch("/guilds/{}/embed", data)
		return GuildEmbed(**resp)

	async def get_invite(self, code):
		resp = await self.http.get("/invites/{}".format(code))
		return Invite(**resp)

	async def delete_invite(self, code):
		resp = await self.http.delete("/invites/{}".format(code))
		return Invite(**resp)


from enum import Enum
import json

CDN_URL = "https://cdn.discordapp.com/"

class ChannelType(Enum):
    text = 0
    private = 1
    voice = 2
    group = 3

    def __str__(self):
        return self.name

class FilterLevel(Enum):
    disabled = 0
    members_without_roles = 1
    all_members = 2
    no = None

class MessageType(Enum):
    default = 0
    recipient_add = 1
    recipient_remove = 2
    call = 3
    channel_name_change = 4
    channel_icon_change = 5
    pins_add = 6
    guild_member_join = 7
    no = None

class ServerRegion(Enum):
    # I should update this.....
    us_west = 'us-west'
    us_east = 'us-east'
    us_south = 'us-south'
    us_central = 'us-central'
    eu_west = 'eu-west'
    eu_central = 'eu-central'
    singapore = 'singapore'
    london = 'london'
    sydney = 'sydney'
    amsterdam = 'amsterdam'
    frankfurt = 'frankfurt'
    brazil = 'brazil'
    vip_us_east = 'vip-us-east'
    vip_us_west = 'vip-us-west'
    vip_amsterdam = 'vip-amsterdam'
    no = None

    def __str__(self):
        return self.value

class VerificationLevel(Enum):
    none = 0
    low = 1
    medium = 2
    high = 3
    table_flip = 3
    very_high = 4
    no = None

    def __str__(self):
        return self.name

def get_from_kwargs(kwargs, key):
    if not kwargs:
        return None
    if key not in kwargs:
        return None
    return kwargs[key]

class Emoji:
    id = None
    name = None
    roles = None
    require_colons = None
    managed = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.name = get_from_kwargs(kwargs, "name")
            self.roles = get_from_kwargs(kwargs, "roles")
            self.require_colons = get_from_kwargs(kwargs, "require_colons")
            self.managed = get_from_kwargs(kwargs, "managed")

def get_emojis(emojis):
    if not emojis:
        return []
    realE = []
    for e in emojis:
        realE.append(Emoji(**e))
    return realE

class Channel:
    id = None
    type = None
    guild_id = None
    position = None
    permission_overwrites = None
    name = None
    topic = None
    last_message_id = None
    bitrate = None
    user_limit = None
    recipients = None
    icon = None
    owner_id = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            type = get_from_kwargs(kwargs, "type")
            if type:
                self.type = ChannelType(int(type))
            self.guild_id = get_from_kwargs(kwargs, "guild_id")
            self.position = get_from_kwargs(kwargs, "position")
            self.permission_overwrites = get_overwrites(get_from_kwargs(kwargs, "permission_overwrites"))
            self.name = get_from_kwargs(kwargs, "name")
            self.topic = get_from_kwargs(kwargs, "topic")
            self.last_message_id = get_from_kwargs(kwargs, "last_message_id")
            self.bitrate = get_from_kwargs(kwargs, "bitrate")
            self.user_limit = get_from_kwargs(kwargs, "user_limit")
            self.recipients = get_from_kwargs(kwargs, "recipients")
            self.icon = get_from_kwargs(kwargs, "icon")
            self.owner_id = get_from_kwargs(kwargs, "owner_id")

def get_channels(chans):
    if not chans:
        return []
    realC = []
    for c in chans:
        realC.append(Channel(**c))
    return realC

class Message:
    id = None
    channel_id = None
    author = None
    content = None
    timestamp = None
    edited_timestamp = None
    tts = None
    mention_everyone = None
    mentions = None
    mention_roles = None
    attachments = None
    embeds = None
    reactions = None
    nonce = None
    pinned = None
    type = None
    webhook_id = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.channel_id = get_from_kwargs(kwargs, "channel_id")
            self.author = get_from_kwargs(kwargs, "author")
            self.content = get_from_kwargs(kwargs, "content")
            self.timestamp = get_from_kwargs(kwargs, "timestamp")
            self.edited_timestamp = get_from_kwargs(kwargs, "edited_timestamp")
            self.tts = get_from_kwargs(kwargs, "tts")
            self.mention_everyone = get_from_kwargs(kwargs, "mention_everyone")
            self.mentions = get_from_kwargs(kwargs, "mentions")
            self.mention_roles = get_from_kwargs(kwargs, "mention_roles")
            self.attachments = get_from_kwargs(kwargs, "attachments")
            self.embeds = get_from_kwargs(kwargs, "embeds")
            self.reactions = get_from_kwargs(kwargs, "reactions")
            self.nonce = get_from_kwargs(kwargs, "nonce")
            self.pinned = get_from_kwargs(kwargs, "pinned")
            self.webhook_id = get_from_kwargs(kwargs, "webhook_id")
            type = get_from_kwargs(kwargs, "type")
            if type:
                self.type = MessageType(int(type))

class Reaction:
    count = None
    me = None
    emoji = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.me = get_from_kwargs(kwargs, "me")
            self.emoji = get_from_kwargs(kwargs, "emoji")

class Overwrite:
    id = None
    type = None
    allow = None
    deny = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.type = get_from_kwargs(kwargs, "type")
            self.allow = get_from_kwargs(kwargs, "allow")
            self.deny = get_from_kwargs(kwargs, "deny")

class Role:
    id = None
    name = None
    color = None
    hoist = None
    position = None
    permissions = None
    managed = None
    mentionable = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.name = get_from_kwargs(kwargs, "name")
            self.color = get_from_kwargs(kwargs, "color")
            self.hoist = get_from_kwargs(kwargs, "hoist")
            self.position = get_from_kwargs(kwargs, "position")
            self.permissions = get_from_kwargs(kwargs, "permissions")
            self.managed = get_from_kwargs(kwargs, "managed")
            self.mentionable = get_from_kwargs(kwargs, "mentionable")

def get_roles(roles):
    if not roles:
        return []
    realR = []
    for r in roles:
        realR.append(Role(**r))
    return realR

def get_overwrites(overwrites):
    if not overwrites:
        return []
    realO = []
    for o in overwrites:
        realO.append(Overwrite(**o))
    return realO

class EmbedThumbnail:
    url = None
    proxy_url =  None
    height = None
    width = None

    def __init__(self, **kwargs):
        if kwargs:
            self.url = get_from_kwargs(kwargs, "url")
            self.proxy_url = get_from_kwargs(kwargs, "proxy_url")
            self.height = get_from_kwargs(kwargs, "height")
            self.width = get_from_kwargs(kwargs, "width")

class EmbedVideo:
    url = None
    height = None
    width = None

    def __init__(self, **kwargs):
        if kwargs:
            self.url = get_from_kwargs(kwargs, "url")
            self.height = get_from_kwargs(kwargs, "height")
            self.width = get_from_kwargs(kwargs, "width")

class EmbedImage:
    url = None
    proxy_url =  None
    height = None
    width = None

    def __init__(self, **kwargs):
        if kwargs:
            self.url = get_from_kwargs(kwargs, "url")
            self.proxy_url = get_from_kwargs(kwargs, "proxy_url")
            self.height = get_from_kwargs(kwargs, "height")
            self.width = get_from_kwargs(kwargs, "width")

class EmbedProvider:
    name = None
    url =  None

    def __init__(self, **kwargs):
        if kwargs:
            self.name = get_from_kwargs(kwargs, "name")
            self.url = get_from_kwargs(kwargs, "url")


class EmbedAuthor:
    name = None
    url =  None
    icon_url =  None
    proxy_icon_url =  None

    def __init__(self, **kwargs):
        if kwargs:
            self.name = get_from_kwargs(kwargs, "name")
            self.url = get_from_kwargs(kwargs, "url")
            self.icon_url = get_from_kwargs(kwargs, "icon_url")
            self.proxy_icon_url = get_from_kwargs(kwargs, "proxy_icon_url")

class EmbedFooter:
    text = None
    icon_url =  None
    proxy_icon_url =  None

    def __init__(self, **kwargs):
        if kwargs:
            self.text = get_from_kwargs(kwargs, "text")
            self.icon_url = get_from_kwargs(kwargs, "icon_url")
            self.proxy_icon_url = get_from_kwargs(kwargs, "proxy_icon_url")

class EmbedField:
    name = None
    value =  None
    inline =  None

    def __init__(self, **kwargs):
        if kwargs:
            self.name = get_from_kwargs(kwargs, "name")
            self.value = get_from_kwargs(kwargs, "value")
            self.inline = get_from_kwargs(kwargs, "inline")

def get_fields(fields):
    if not fields:
        return []
    realFields = []
    for field in fields:
        realFields.append(EmbedField(**field))
    return realFields

class Embed:
    title = None
    type = None
    description = None
    url = None
    timestamp = None
    color = None
    footer = None
    image = None
    thumbnail = None
    video = None
    provider = None
    author = None
    fields = None

    def __init__(self, **kwargs):
        if kwargs:
            self.title = get_from_kwargs(kwargs, "title")
            self.type = get_from_kwargs(kwargs, "type")
            self.description = get_from_kwargs(kwargs, "description")
            self.url = get_from_kwargs(kwargs, "url")
            self.timestamp = get_from_kwargs(kwargs, "timestamp")
            self.color = get_from_kwargs(kwargs, "color")
            self.footer = EmbedFooter(**get_from_kwargs(kwargs, "footer"))
            self.image = EmbedImage(**get_from_kwargs(kwargs, "image"))
            self.thumbnail = EmbedThumbnail(**get_from_kwargs(kwargs, "thumbnail"))
            self.video = EmbedVideo(**get_from_kwargs(kwargs, "video"))
            self.provider = EmbedProvider(**get_from_kwargs(kwargs, "provider"))
            self.author = EmbedAuthor(**get_from_kwargs(kwargs, "author"))
            self.fields = get_fields(**get_from_kwargs(kwargs, "fields"))

    def __str__(self):
        return json.dumps(self.__dict__)

class Attachment:
    id = None
    filename =  None
    size =  None
    url =  None
    proxy_url =  None
    height =  None
    width =  None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.filename = get_from_kwargs(kwargs, "filename")
            self.size = get_from_kwargs(kwargs, "size")
            self.url = get_from_kwargs(kwargs, "url")
            self.proxy_url = get_from_kwargs(kwargs, "proxy_url")
            self.height = get_from_kwargs(kwargs, "height")
            self.width = get_from_kwargs(kwargs, "width")

class Guild:
    id = None
    name = None
    icon =  None
    splash =  None
    owner_id =  None
    region =  None
    afk_channel_id =  None
    afk_timeout =  None
    embed_enabled =  None
    embed_channel_id =  None
    verification_level =  None
    default_message_notifications =  None
    explicit_content_filter =  None
    roles =  None
    emojis =  None
    features =  None
    mfa_level =  None
    widget_enabled =  None
    widget_channel_id =  None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.name = get_from_kwargs(kwargs, "name")
            self.icon = "{}icons/{}/{}.png".format(CDN_URL, self.id, get_from_kwargs(kwargs, "icon"))
            self.splash = "{}splashes/{}/{}.png".format(CDN_URL, self.id, get_from_kwargs(kwargs, "splash"))
            self.owner_id = get_from_kwargs(kwargs, "owner_id")
            self.afk_channel_id = get_from_kwargs(kwargs, "afk_channel_id")
            self.afk_timeout = get_from_kwargs(kwargs, "afk_timeout")
            self.embed_enabled = get_from_kwargs(kwargs, "embed_enabled")
            self.embed_channel_id = get_from_kwargs(kwargs, "embed_channel_id")
            if get_from_kwargs(kwargs, "region"):
                self.region = ServerRegion(get_from_kwargs(kwargs, "region"))
            if get_from_kwargs(kwargs, "verification_level"):
                self.verification_level = VerificationLevel(get_from_kwargs(kwargs, "verification_level"))
            self.default_message_notifications = get_from_kwargs(kwargs, "default_message_notifications")
            if get_from_kwargs(kwargs, "explicit_content_filter"):
                self.explicit_content_filter = FilterLevel(int(get_from_kwargs(kwargs, "explicit_content_filter")))
            self.roles = get_roles(get_from_kwargs(kwargs, "roles"))
            self.emojis = get_emojis(get_from_kwargs(kwargs, "emojis"))
            self.features = get_from_kwargs(kwargs, "features")
            self.mfa_level = get_from_kwargs(kwargs, "mfa_level")
            self.widget_enabled = get_from_kwargs(kwargs, "widget_enabled")
            self.widget_channel_id = get_from_kwargs(kwargs, "widget_channel_id")

class GuildEmbed:
    enabled = None
    channel_id = None

    def __init__(self, **kwargs):
        if kwargs:
            self.enabled = get_from_kwargs(kwargs, "enabled")
            self.channel_id = get_from_kwargs(kwargs, "channel_id")

class User:
    id = None
    username = None
    discriminator = None
    avatar = None
    bot = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.username = get_from_kwargs(kwargs, "username")
            self.discriminator = get_from_kwargs(kwargs, "discriminator")
            self.avatar = get_from_kwargs(kwargs, "avatar")
            self.bot = get_from_kwargs(kwargs, "bot")

def get_recipients(recipients):
    if not recipients:
        return []
    realR = []
    for r in recipients:
        realR.append(User(**r))
    return realR

class GuildMember(User):
    nick = None
    roles = None
    joined_at = None
    deaf = None
    mute = None

    def __init__(self, **kwargs):
        if kwargs:
            super(**get_from_kwargs(kwargs, "user"))
            self.nick = get_from_kwargs(kwargs, "nick")
            self.roles = get_from_kwargs(kwargs, "roles")
            self.joined_at = get_from_kwargs(kwargs, "joined_at")
            self.deaf = get_from_kwargs(kwargs, "deaf")
            self.mute = get_from_kwargs(kwargs, "mute")

class Integration:
    id = None
    name = None
    type = None
    enabled = None
    syncing = None
    role_id = None
    expire_behavior = None
    expire_grace_period = None
    user = None
    account = None
    synced_at = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.name = get_from_kwargs(kwargs, "name")
            self.type = get_from_kwargs(kwargs, "type")
            self.enabled = get_from_kwargs(kwargs, "enabled")
            self.syncing = get_from_kwargs(kwargs, "syncing")
            self.role_id = get_from_kwargs(kwargs, "role_id")
            self.syncing = get_from_kwargs(kwargs, "syncing")
            self.expire_behavior = get_from_kwargs(kwargs, "expire_behavior")
            self.expire_grace_period = get_from_kwargs(kwargs, "expire_grace_period")
            self.user = User(**get_from_kwargs(kwargs, "user"))
            self.account = IntegrationAccount(**get_from_kwargs(kwargs, "account"))
            self.synced_at = get_from_kwargs(kwargs, "synced_at")

class IntegrationAccount:
    id = None
    name = None

    def __init__(self, **kwargs):
        if kwargs:
            self.id = get_from_kwargs(kwargs, "id")
            self.name = get_from_kwargs(kwargs, "name")

class Ban:
    reason = None
    user = None

    def __init__(self, **kwargs):
        if kwargs:
            self.reason = get_from_kwargs(kwargs, "reason")
            self.user = User(**get_from_kwargs(kwargs, "user"))

class Invite:
    code = None
    guild = None
    channel = None

    def __init__(self, **kwargs):
        if kwargs:
            self.code = get_from_kwargs(kwargs, "code")
            self.guild = Guild(**get_from_kwargs(kwargs, "guild"))
            self.channel = Channel(**get_from_kwargs(kwargs, "channel"))



from setuptools import setup
import re, os, sys
from restcord.vars import __version__

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

requirements = []
with open('requirements.txt') as f:
  requirements = f.read().splitlines()

version = __version__
if not version:
    raise RuntimeError('version is not set')

readme = ''
with open('README.rst') as f:
    readme = f.read()

setup(name='Restcord',
      author='JustMaffie',
      version=version,
      packages=['restcord'],
      license='MIT',
      description='Restcord is a rest API wrapper for the Discord API, but this one doesn\'t include WebSockets, this is for people who dont want websockets but only make requests to the api.',
      long_description=readme,
      url="https://github.com/JustMaffie/Restcord",
      include_package_data=True,
      install_requires=requirements,
      classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
      ]
)
